@@ -1,4 +1,4 @@
-# Copyright (c) 2012 CentralNic Ltd. All rights reserved. This program is
+# Copyright (c) 2016 CentralNic Ltd. All rights reserved. This program is
 # free software; you can redistribute it and/or modify it under the same
 # terms as Perl itself.
 # 
@@ -144,7 +144,6 @@ sub new {
 			croak("Frames requested but Net::EPP::Frame isn't available");
 
 		} else {
-			$self->{'parser'} = XML::LibXML->new;
 			$self->{'class'} = 'Net::EPP::Frame';
 
 		}
@@ -154,15 +153,10 @@ sub new {
 			croak("DOM requested but XML::LibXML isn't available");
 
 		} else {
-			$self->{'parser'} = XML::LibXML->new;
 			$self->{'class'} = 'XML::LibXML::Document';
 
 		}
 
-	} else {
-		# for well-formedness checking:
-		$self->{'parser'} = XML::LibXML->new;
-
 	}
 
 	return bless($self, $package);
@@ -309,7 +303,7 @@ sub get_return_value {
 
 	} else {
 		my $document;
-		eval { $document = $self->{'parser'}->parse_string($xml) };
+		eval { $document = $self->parser->parse_string($xml) };
 		if (!defined($document) || $@ ne '') {
 			chomp($@);
 			croak(sprintf("Frame from server wasn't well formed: %s\n\nThe XML looks like this:\n\n%s\n\n", $@, $xml));
@@ -374,7 +368,7 @@ sub send_frame {
 	}
 
 	if ($wfcheck == 1) {
-		eval { $self->{'parser'}->parse_string($xml) };
+		eval { $self->parser->parse_string($xml) };
 		if ($@ ne '') {
 			chomp($@);
 			croak(sprintf("Frame from server wasn't well formed: %s\n\nThe XML looks like this:\n\n%s\n\n", $@, $xml));
@@ -411,7 +405,7 @@ CentralNic Ltd (L<http://www.centralnic.com/>).
 
 =head1 COPYRIGHT
 
-This module is (c) 2012 CentralNic Ltd. This module is free software; you can
+This module is (c) 2016 CentralNic Ltd. This module is free software; you can
 redistribute it and/or modify it under the same terms as Perl itself.
 
 =head1 SEE ALSO
@@ -430,4 +424,10 @@ redistribute it and/or modify it under the same terms as Perl itself.
 
 =cut
 
+sub parser {
+	my $self = shift;
+	$self->{'parser'} = XML::LibXML->new if (!$self->{'parser'});
+	return $self->{'parser'};
+}
+
 1;
