@@ -1,4 +1,4 @@
-# Copyright (c) 2012 CentralNic Ltd. All rights reserved. This program is
+# Copyright (c) 2016 CentralNic Ltd. All rights reserved. This program is
 # free software; you can redistribute it and/or modify it under the same
 # terms as Perl itself.
 # 
@@ -7,6 +7,9 @@ package Net::EPP::Frame::Command::Update::Domain;
 use base qw(Net::EPP::Frame::Command::Update);
 use Net::EPP::Frame::ObjectSpec;
 use strict;
+use warnings;
+
+our $DNSSEC_URN	= 'urn:ietf:params:xml:ns:secDNS-1.1';
 
 =pod
 
@@ -340,9 +343,31 @@ sub remHostObjNS {
 	return 1;
 }
 
+=pod
 
+=head2 DNSSEC methods
 
-=pot
+=cut
+
+sub _get_dnsssec {
+	my $self = shift;
+	my $tag = shift;
+
+	my $el = self->getElementsByTagNameNS($DNSSEC_URN, $tag);
+	return $el if $el;
+
+	my $ext = $self->getNode('extension');
+	$ext = $self->getNode('command')->addNewChild(undef, 'extension')
+	    if not defined $ext;
+
+	my $upd = $ext->addNewChild($DNSSEC_URN, 'secDNS:update');
+	$upd->addNewChild($DNSSEC_URN, 'secDNS:add');
+	$upd->addNewChild($DNSSEC_URN, 'secDNS:rem');
+
+	return $self->_get_dnssec($tag);
+}
+
+=pod
 
 =head1 AUTHOR
 
@@ -350,7 +375,7 @@ CentralNic Ltd (http://www.centralnic.com/), with contributions from United Doma
 
 =head1 COPYRIGHT
 
-This module is (c) 2012 CentralNic Ltd. This module is free software; you can
+This module is (c) 2016 CentralNic Ltd. This module is free software; you can
 redistribute it and/or modify it under the same terms as Perl itself.
 
 =head1 SEE ALSO
