@@ -1,4 +1,4 @@
-# Copyright (c) 2012 CentralNic Ltd. All rights reserved. This program is
+# Copyright (c) 2016 CentralNic Ltd. All rights reserved. This program is
 # free software; you can redistribute it and/or modify it under the same
 # terms as Perl itself.
 # 
@@ -87,6 +87,24 @@ one for C<Net::EPP::Client>, but with the following exceptions:
 
 =item * C<login> can be used to disable automatic logins. If you set it to C<0>, you can manually log in using the C<$epp->_login()> method.
 
+=item * C<objects> is a reference to an array of the EPP object schema
+URIs that the client requires.
+
+=item * C<stdobj> is a flag saying the client only requires the
+standard EPP C<contact-1.0>, C<domain-1.0>, and C<host-1.0> schemas.
+
+=item * If neither C<objects> nor C<stdobj> is specified then the
+client will echo the server's object schema list.
+
+=item * C<extensions> is a reference to an array of the EPP extension
+schema URIs that the client requires.
+
+=item * C<stdext> is a flag saying the client only requires the
+standard EPP C<secDNS-1.1> DNSSEC extension schema.
+
+=item * If neither C<extensions> nor C<stdext> is specified then the
+client will echo the server's extension schema list.
+
 =back
 
 The constructor will establish a connection to the server and retrieve the
@@ -121,6 +139,10 @@ constructor:
 C<Net::EPP::Simple> will fail to connect to the server if the
 certificate is not valid.
 
+You can disable SSL certificate verification by omitting the C<verify>
+argument or setting it to C<undef>. This is strongly discouraged,
+particularly in production environments.
+
 =head3 SSL Cipher Selection
 
 You can restrict the ciphers that you will use to connect to the server
@@ -222,6 +244,10 @@ sub new {
 	$self->{ca_file}	= $params{ca_file};
 	$self->{ca_path}	= $params{ca_path};
 	$self->{ciphers}	= $params{ciphers};
+	$self->{objects}	= $params{objects};
+	$self->{stdobj}		= $params{stdobj};
+	$self->{extensions}	= $params{extensions};
+	$self->{stdext}		= $params{stdext};
 
 	bless($self, $package);
 
@@ -281,9 +307,13 @@ sub _connect {
 
 	if (defined($self->{ssl}) && defined($self->{verify})) {
 		$self->debug('configuring server verification');
-		$params{SSL_verify_mode}	= 0x01;
+		$params{SSL_verify_mode}	= 1;
 		$params{SSL_ca_file}		= $self->{ca_file};
 		$params{SSL_ca_path}		= $self->{ca_path};
+
+	} elsif (defined($self->{ssl})) {
+		$params{SSL_verify_mode} = 0;
+
 	}
 
 	$self->debug(sprintf('Attempting to connect to %s:%d', $self->{host}, $self->{port}));
@@ -344,7 +374,7 @@ sub _login {
 		$self->debug(sprintf('%04d: %s', $Code, $Message));
 
 		if ($Code > 1999) {
-			$Error = "Error logging in (response code $Code)";
+			$Error = "Error logging in (response code $Code, message $Message)";
 			return undef;
 
 		} else {
@@ -355,6 +385,17 @@ sub _login {
 	}
 }
 
+sub _get_option_uri_list {
+	my $self = shift;
+	my $tag = shift;
+	my $list = [];
+	my $elems = $self->{greeting}->getElementsByTagNameNS(EPP_XMLNS, $tag);
+	while (my $elem = $elems->shift) {
+		push @$list, $elem->firstChild->data;
+	}
+	return $list;
+}
+
 sub _prepare_login_frame {
 	my $self = shift;
 
@@ -366,22 +407,20 @@ sub _prepare_login_frame {
 	$login->version->appendText($self->{greeting}->getElementsByTagNameNS(EPP_XMLNS, 'version')->shift->firstChild->data);
 	$login->lang->appendText($self->{greeting}->getElementsByTagNameNS(EPP_XMLNS, 'lang')->shift->firstChild->data);
 
-	my $objects = $self->{greeting}->getElementsByTagNameNS(EPP_XMLNS, 'objURI');
-	while (my $object = $objects->shift) {
-		my $el = $login->createElement('objURI');
-		$el->appendText($object->firstChild->data);
-		$login->svcs->appendChild($el);
-	}
-	$objects = $self->{greeting}->getElementsByTagNameNS(EPP_XMLNS, 'extURI');
-	my $svcext;
-	if ($objects->size) {
-		$svcext = $login->createElement('svcExtension');
+	my $objects = $self->{objects};
+	$objects = [map { (Net::EPP::Frame::ObjectSpec->spec($_))[1] }
+		    qw(contact domain host)] if $self->{stdobj};
+	$objects = _get_option_uri_list($self,'objURI') if not $objects;
+	$login->svcs->appendTextChild('objURI', $_) for @$objects;
+
+	my $extensions = $self->{extensions};
+	$extensions = [map { (Net::EPP::Frame::ObjectSpec->spec($_))[1] }
+		       qw(secDNS)] if $self->{stdext};
+	$extensions = _get_option_uri_list($self,'extURI') if not $extensions;
+	if (@$extensions) {
+		my $svcext = $login->createElement('svcExtension');
 		$login->svcs->appendChild($svcext);
-	}
-	while (my $object = $objects->shift) {
-		my $el = $login->createElement('extURI');
-		$el->appendText($object->firstChild->data);
-		$svcext->appendChild($el);
+		$svcext->appendTextChild('extURI', $_) for @$extensions;
 	}
 	return $login;
 }
@@ -606,8 +645,9 @@ sub parse_object_info {
 	my $infData = $response->getNode((Net::EPP::Frame::ObjectSpec->spec($type))[1], 'infData');
 
 	if ($type eq 'domain') {
-		return $self->_domain_infData_to_hash($infData);
-
+		# secDNS extension only applies to domain objects
+		my $secinfo = $response->getNode((Net::EPP::Frame::ObjectSpec->spec('secDNS'))[1], 'infData');
+		return $self->_domain_infData_to_hash($infData, $secinfo);
 	} elsif ($type eq 'contact') {
 		return $self->_contact_infData_to_hash($infData);
 
@@ -702,7 +742,7 @@ the C<type> attribute is optional.
 =cut
 
 sub _domain_infData_to_hash {
-	my ($self, $infData) = @_;
+	my ($self, $infData, $secinfo) = @_;
 
 	my $hash = $self->_get_common_properties_from_infData($infData, 'registrant', 'name', 'exDate');
 
@@ -756,6 +796,24 @@ sub _domain_infData_to_hash {
 		$hash->{authInfo} = $pw->shift->textContent if ($pw->size == 1);
 	}
 
+	if (defined $secinfo) {
+		if (my $maxSigLife = $secinfo->getElementsByLocalName('maxSigLife')) {
+			$hash->{maxSigLife} = $maxSigLife->shift->textContent;
+		}
+		my $dslist = $secinfo->getElementsByTagName('secDNS:dsData');
+		while (my $ds = $dslist->shift) {
+			my @ds = map { $ds->getElementsByLocalName($_)->string_value() }
+			    qw(keyTag alg digestType digest);
+			push @{ $hash->{DS} }, "@ds";
+		}
+		my $keylist = $secinfo->getElementsByLocalName('keyData');
+		while (my $key = $keylist->shift) {
+			my @key = map { $key->getElementsByLocalName($_)->string_value() }
+			    qw(flags protocol alg pubKey);
+			push @{ $hash->{DNSKEY} }, "@key";
+		}
+	}
+
 	return $hash;
 }
 
@@ -883,6 +941,7 @@ sub _contact_infData_to_hash {
 		if ($addrs->size == 1) {
 			my $addr = $addrs->shift;
 			foreach my $child ($addr->childNodes) {
+				next if (XML::LibXML::XML_ELEMENT_NODE != $child->nodeType);
 				if ($child->localName eq 'street') {
 					push(@{$ref->{addr}->{$child->localName}}, $child->textContent);
 
@@ -962,6 +1021,7 @@ sub _transfer_request {
 		$frame->setOp($op);
 		if ($type eq 'domain') {
 			$frame->setDomain($identifier);
+			$frame->setPeriod(int($period)) if ($op eq 'request');
 
 		} elsif ($type eq 'contact') {
 			$frame->setContact($identifier);
@@ -972,7 +1032,6 @@ sub _transfer_request {
 			$frame->setAuthInfo($authInfo) if ($authInfo ne '');
 		}
 
-		$frame->setPeriod(int($period)) if ($op eq 'request');
 	}
 
 	my $response = $self->_request($frame);
@@ -1823,9 +1882,9 @@ sub request {
 
 	$frame->clTRID->appendText(sha1_hex(ref($self).time().$$)) if (UNIVERSAL::isa($frame, 'Net::EPP::Frame::Command'));
 
-	$self->debug(sprintf('sending a %s to the server', ref($frame)));
+	$self->debug(sprintf('sending a %s to the server', ref($frame) || (-e $frame ? 'file' : 'string')));
 	if (UNIVERSAL::isa($frame, 'XML::LibXML::Document')) {
-		map { $self->debug('C: '.$_) } split(/\n/, $frame->toString(1));
+		map { $self->debug('C: '.$_) } split(/\n/, $frame->toString(2));
 
 	} else {
 		map { $self->debug('C: '.$_) } split(/\n/, $frame);
@@ -1834,7 +1893,7 @@ sub request {
 
 	my $response = $self->SUPER::request($frame);
 
-	map { $self->debug('S: '.$_) } split(/\n/, $response->toString(1)) if (UNIVERSAL::isa($response, 'XML::LibXML::Document'));
+	map { $self->debug('S: '.$_) } split(/\n/, $response->toString(2)) if (UNIVERSAL::isa($response, 'XML::LibXML::Document'));
 
 	return $response;
 }
@@ -1995,7 +2054,7 @@ CentralNic Ltd (L<http://www.centralnic.com/>).
 
 =head1 Copyright
 
-This module is (c) 2012 CentralNic Ltd. This module is free software; you can
+This module is (c) 2016 CentralNic Ltd. This module is free software; you can
 redistribute it and/or modify it under the same terms as Perl itself.
 
 =head1 SEE ALSO
